@@ -1,9 +1,10 @@
 #!/usr/bin/env python
 from __future__ import with_statement
 import daemon
-import lockfile
+from lockfile.pidlockfile import PIDLockFile
 import sys
 import signal
+import os
 from rpyc.utils.server import ThreadedServer, ForkingServer
 from rpyc.core.service import SlaveService
 from rpyc.lib import setup_logger
@@ -13,12 +14,18 @@ except ImportError:
     from ConfigParser import ConfigParser
 
 server = None
+cur_dir = os.getcwd()
+bin_dir = os.path.dirname(__file__)
+
 
 def start():
     global server
 
     conf = ConfigParser()
-    conf.read('rpycd.conf')
+    conf.read([
+        os.path.join(cur_dir, 'rpycd.conf'),
+        os.path.join(bin_dir, 'rpycd.conf'),    # later files trump earlier ones
+    ])
 
     mode = conf.get("rpycd", "mode").lower()
     if mode == "threaded":
@@ -28,11 +35,14 @@ def start():
     else:
         raise ValueError("Invalid mode %r" % (mode,))
 
-    setup_logger(conf.getboolean("rpycd", "quiet"), conf.get("rpycd", "logfile"))
+    quiet = conf.getboolean("rpycd", "quiet")
+    logfile = os.path.join(cur_dir, conf.get("rpycd", "logfile"))
+    setup_logger(quiet, logfile)
 
     server = factory(SlaveService, hostname = conf.get("rpycd", "host"),
         port = conf.getint("rpycd", "port"), reuse_addr = True)
     server.start()
+    server.serve_all()
 
 def reload(*args):
     server.close()
@@ -44,8 +54,8 @@ def stop(*args):
 
 
 if __name__ == "__main__":
+    pid_file = os.path.join(cur_dir, 'rpycd.pid')
     with daemon.DaemonContext(
-            pidfile = lockfile.FileLock('/var/run/rpydc.pid'),
+            pidfile = PIDLockFile(pid_file),
             signal_map = {signal.SIGTERM: stop, signal.SIGHUP: reload}):
         start()
-
