@@ -2,17 +2,18 @@
 The RPyC protocol
 """
 import sys
-import weakref
 import itertools
 import socket
 import time
 import gc
 
-from threading import Lock, RLock, Event, Thread
-from rpyc.lib.compat import pickle, next, is_py3k, maxint, select_error
+from threading import Lock, Condition
+from rpyc.lib import spawn, Timeout
+from rpyc.lib.compat import (pickle, next, is_py3k, maxint, select_error,
+                             acquire_lock)
 from rpyc.lib.colls import WeakValueDict, RefCountingColl
 from rpyc.core import consts, brine, vinegar, netref
-from rpyc.core.async import AsyncResult
+from rpyc.core.async_ import AsyncResult
 
 class PingError(Exception):
     """The exception raised should :func:`Connection.ping` fail"""
@@ -121,48 +122,39 @@ Parameter                                Default value     Description
 
 _connection_id_generator = itertools.count(1)
 
+
 class Connection(object):
     """The RPyC *connection* (AKA *protocol*).
 
-    :param service: the :class:`Service <rpyc.core.service.Service>` to expose
-    :param channel: the :class:`Channel <rpyc.core.channel.Channel>` over which messages are passed
+    :param root: the :class:`~rpyc.core.service.Service` object to expose
+    :param channel: the :class:`~rpyc.core.channel.Channel` over which messages are passed
     :param config: the connection's configuration dict (overriding parameters
                    from the :data:`default configuration <DEFAULT_CONFIG>`)
-    :param _lazy: whether or not to initialize the service with the creation of
-                  the connection. Default is True. If set to False, you will
-                  need to call :func:`_init_service` manually later
     """
 
-    def __init__(self, service, channel, config = {}, _lazy = False):
+    def __init__(self, root, channel, config={}):
         self._closed = True
         self._config = DEFAULT_CONFIG.copy()
         self._config.update(config)
         if self._config["connid"] is None:
             self._config["connid"] = "conn%d" % (next(_connection_id_generator),)
 
+        self._HANDLERS = self._request_handlers()
         self._channel = channel
         self._seqcounter = itertools.count()
         self._recvlock = Lock()
         self._sendlock = Lock()
-        self._sync_replies = {}
-        self._sync_lock = RLock()
-        self._sync_event = Event()
-        self._async_callbacks = {}
+        self._recv_event = Condition()
+        self._request_callbacks = {}
         self._local_objects = RefCountingColl()
         self._last_traceback = None
         self._proxy_cache = WeakValueDict()
         self._netref_classes_cache = {}
         self._remote_root = None
         self._send_queue = []
-        self._local_root = service(weakref.proxy(self))
-        if not _lazy:
-            self._init_service()
+        self._local_root = root
         self._closed = False
 
-
-    def _init_service(self):
-        self._local_root.on_connect()
-
     def __del__(self):
         self.close()
     def __enter__(self):
@@ -181,9 +173,8 @@ class Connection(object):
             return
         self._closed = True
         self._channel.close()
-        self._local_root.on_disconnect()
-        self._sync_replies.clear()
-        self._async_callbacks.clear()
+        self._local_root.on_disconnect(self)
+        self._request_callbacks.clear()
         self._local_objects.clear()
         self._proxy_cache.clear()
         self._netref_classes_cache.clear()
@@ -192,6 +183,7 @@ class Connection(object):
         self._local_root = None
         #self._seqcounter = None
         #self._config.clear()
+        del self._HANDLERS
 
     def close(self, _catchall = True):
         """closes the connection, releasing all held resources"""
@@ -226,6 +218,7 @@ class Connection(object):
         :param timeout: the maximal time to wait for echo
 
         :raises: :class:`PingError` if the echoed data does not match
+        :raises: :class:`EOFError` if the remote host closes the connection
         """
         if data is None:
             data = "abcdefghijklmnopqrstuvwxyz" * 20
@@ -267,17 +260,6 @@ class Connection(object):
             finally:
                 self._sendlock.release()
 
-    def _send_request(self, seq, handler, args):
-        self._send(consts.MSG_REQUEST, seq, (handler, self._box(args)))
-
-    def _send_reply(self, seq, obj):
-        self._send(consts.MSG_REPLY, seq, self._box(obj))
-
-    def _send_exception(self, seq, exctype, excval, exctb):
-        exc = vinegar.dump(exctype, excval, exctb,
-            include_local_traceback = self._config["include_local_traceback"])
-        self._send(consts.MSG_EXCEPTION, seq, exc)
-
     #
     # boxing
     #
@@ -288,7 +270,7 @@ class Connection(object):
             return consts.LABEL_VALUE, obj
         if type(obj) is tuple:
             return consts.LABEL_TUPLE, tuple(self._box(item) for item in obj)
-        elif isinstance(obj, netref.BaseNetref) and obj.____conn__() is self:
+        elif isinstance(obj, netref.BaseNetref) and obj.____conn__ is self:
             return consts.LABEL_LOCAL_REF, obj.____oid__
         else:
             self._local_objects.add(obj)
@@ -335,7 +317,7 @@ class Connection(object):
             info = self.sync_request(consts.HANDLE_INSPECT, oid)
             cls = netref.class_factory(clsname, modname, info)
             self._netref_classes_cache[typeinfo] = cls
-        return cls(weakref.ref(self), oid)
+        return cls(self, oid)
 
     #
     # dispatching
@@ -356,81 +338,38 @@ class Connection(object):
                 raise
             if t is KeyboardInterrupt and self._config["propagate_KeyboardInterrupt_locally"]:
                 raise
-            self._send_exception(seq, t, v, tb)
+            self._send(consts.MSG_EXCEPTION, seq, self._box_exc(t, v, tb))
         else:
-            self._send_reply(seq, res)
+            self._send(consts.MSG_REPLY, seq, self._box(res))
 
-    def _dispatch_reply(self, seq, raw):
-        obj = self._unbox(raw)
-        if seq in self._async_callbacks:
-            self._async_callbacks.pop(seq)(False, obj)
-        else:
-            self._sync_replies[seq] = (False, obj)
+    def _box_exc(self, typ, val, tb):
+        return vinegar.dump(typ, val, tb, include_local_traceback=
+                            self._config["include_local_traceback"])
 
-    def _dispatch_exception(self, seq, raw):
-        obj = vinegar.load(raw,
+    def _unbox_exc(self, raw):
+        return vinegar.load(raw,
             import_custom_exceptions = self._config["import_custom_exceptions"],
             instantiate_custom_exceptions = self._config["instantiate_custom_exceptions"],
             instantiate_oldstyle_exceptions = self._config["instantiate_oldstyle_exceptions"])
-        if seq in self._async_callbacks:
-            self._async_callbacks.pop(seq)(True, obj)
-        else:
-            self._sync_replies[seq] = (True, obj)
 
     #
     # serving
     #
-    def _recv(self, timeout, wait_for_lock):
-        if not self._recvlock.acquire(wait_for_lock):
-            return None
-        try:
-            if self._channel.poll(timeout):
-                data = self._channel.recv()
-            else:
-                data = None
-        except EOFError:
-            self.close()
-            raise
-        finally:
-            self._recvlock.release()
-        return data
 
     def _dispatch(self, data):
         msg, seq, args = brine.load(data)
         if msg == consts.MSG_REQUEST:
             self._dispatch_request(seq, args)
         elif msg == consts.MSG_REPLY:
-            self._dispatch_reply(seq, args)
+            obj = self._unbox(args)
+            self._request_callbacks.pop(seq)(False, obj)
         elif msg == consts.MSG_EXCEPTION:
-            self._dispatch_exception(seq, args)
+            obj = self._unbox_exc(args)
+            self._request_callbacks.pop(seq)(True, obj)
         else:
             raise ValueError("invalid message type: %r" % (msg,))
 
-    def sync_recv_and_dispatch(self, timeout, wait_for_lock):
-        # lock or wait for signal
-        if self._sync_lock.acquire(False):
-            try:
-                self._sync_event.clear()
-                data = self._recv(timeout, wait_for_lock = False)
-                if not data:
-                    return False
-                self._dispatch(data)
-                return True
-            finally:
-                self._sync_lock.release()
-                self._sync_event.set()
-        else:
-            self._sync_event.wait()
-
-    def poll(self, timeout = 0):
-        """Serves a single transaction, should one arrives in the given
-        interval. Note that handling a request/reply may trigger nested
-        requests, which are all part of a single transaction.
-
-        :returns: ``True`` if a transaction was served, ``False`` otherwise"""
-        return self.sync_recv_and_dispatch(timeout, wait_for_lock=False)
-
-    def serve(self, timeout = 1):
+    def serve(self, timeout=1, wait_for_lock=True):
         """Serves a single request or reply that arrives within the given
         time frame (default is 1 sec). Note that the dispatching of a request
         might trigger multiple (nested) requests, thus this function may be
@@ -439,7 +378,32 @@ class Connection(object):
         :returns: ``True`` if a request or reply were received, ``False``
                   otherwise.
         """
-        return self.sync_recv_and_dispatch(timeout, wait_for_lock=True)
+        timeout = Timeout(timeout)
+        with self._recv_event:
+            if not self._recvlock.acquire(False):
+                return (wait_for_lock and
+                        self._recv_event.wait(timeout.timeleft()))
+        try:
+            data = self._channel.poll(timeout) and self._channel.recv()
+            if not data:
+                return False
+        except EOFError:
+            self.close()
+            raise
+        finally:
+            self._recvlock.release()
+            with self._recv_event:
+                self._recv_event.notify_all()
+        self._dispatch(data)
+        return True
+
+    def poll(self, timeout = 0):
+        """Serves a single transaction, should one arrives in the given
+        interval. Note that handling a request/reply may trigger nested
+        requests, which are all part of a single transaction.
+
+        :returns: ``True`` if a transaction was served, ``False`` otherwise"""
+        return self.serve(timeout, False)
 
     def serve_all(self):
         """Serves all requests and replies for as long as the connection is
@@ -456,6 +420,8 @@ class Connection(object):
             self.close()
 
     def serve_threaded(self, thread_count=10):
+        """Serves all requests and replies for as long as the connection is
+        alive."""
         def _thread_target():
             try:
                 while True:
@@ -466,16 +432,9 @@ class Connection(object):
             except EOFError:
                 pass
 
-        threads = []
-
-        """Serves all requests and replies for as long as the connection is
-        alive."""
         try:
-            for _ in range(thread_count):
-                thread = Thread(target=_thread_target)
-                thread.daemon = True
-                thread.start()
-                threads.append(thread)
+            threads = [spawn(_thread_target)
+                       for _ in range(thread_count)]
 
             for thread in threads:
                 thread.join()
@@ -488,16 +447,13 @@ class Connection(object):
         :returns: ``True`` if at least a single transaction was served, ``False`` otherwise
         """
         at_least_once = False
-        t0 = time.time()
-        duration = timeout
+        timeout = Timeout(timeout)
         try:
             while True:
-                if self.poll(duration):
+                if self.poll(timeout):
                     at_least_once = True
-                if timeout is not None:
-                    duration = t0 + timeout - time.time()
-                    if duration < 0:
-                        break
+                if timeout.expired():
+                    break
         except EOFError:
             pass
         return at_least_once
@@ -511,39 +467,28 @@ class Connection(object):
         :raises: any exception that the requets may be generated
         :returns: the result of the request
         """
-        seq = self._get_seq_id()
-        self._send_request(seq, handler, args)
-
         timeout = self._config["sync_request_timeout"]
-        while seq not in self._sync_replies:
-            self.sync_recv_and_dispatch(timeout, True)
-
-        isexc, obj = self._sync_replies.pop(seq)
-        if isexc:
-            raise obj
-        else:
-            return obj
+        return self.async_request(handler, *args, timeout=timeout).value
 
     def _async_request(self, handler, args = (), callback = (lambda a, b: None)):
         seq = self._get_seq_id()
-        self._async_callbacks[seq] = callback
+        self._request_callbacks[seq] = callback
         try:
-            self._send_request(seq, handler, args)
+            self._send(consts.MSG_REQUEST, seq, (handler, self._box(args)))
         except:
-            if seq in self._async_callbacks:
-                del self._async_callbacks[seq]
+            self._request_callbacks.pop(seq, None)
             raise
 
     def async_request(self, handler, *args, **kwargs):
         """Send an asynchronous request (does not wait for it to finish)
 
-        :returns: an :class:`rpyc.core.async.AsyncResult` object, which will
+        :returns: an :class:`rpyc.core.async_.AsyncResult` object, which will
                   eventually hold the result (or exception)
         """
         timeout = kwargs.pop("timeout", None)
         if kwargs:
             raise TypeError("got unexpected keyword argument(s) %s" % (list(kwargs.keys()),))
-        res = AsyncResult(weakref.proxy(self))
+        res = AsyncResult(self)
         self._async_request(handler, args, res)
         if timeout is not None:
             res.set_expiry(timeout)
@@ -559,23 +504,25 @@ class Connection(object):
     #
     # attribute access
     #
-    def _check_attr(self, obj, name):
-        if self._config["allow_exposed_attrs"]:
-            if name.startswith(self._config["exposed_prefix"]):
-                name2 = name
-            else:
-                name2 = self._config["exposed_prefix"] + name
-            if hasattr(obj, name2):
-                return name2
-        if self._config["allow_all_attrs"]:
+    def _check_attr(self, obj, name, perm):
+        config = self._config
+        if not config[perm]:
+            raise AttributeError("cannot access %r" % (name,))
+        prefix = config["allow_exposed_attrs"] and config["exposed_prefix"]
+        plain = (config["allow_all_attrs"] or
+                 config["allow_exposed_attrs"] and name.startswith(prefix) or
+                 config["allow_safe_attrs"] and name in config["safe_attrs"] or
+                 config["allow_public_attrs"] and not name.startswith("_"))
+        has_exposed = prefix and hasattr(obj, prefix+name)
+        if plain and (not has_exposed or hasattr(obj, name)):
             return name
-        if self._config["allow_safe_attrs"] and name in self._config["safe_attrs"]:
-            return name
-        if self._config["allow_public_attrs"] and not name.startswith("_"):
-            return name
-        return False
+        if has_exposed:
+            return prefix+name
+        if plain:
+            return name         # chance for better traceback
+        raise AttributeError("cannot access %r" % (name,))
 
-    def _access_attr(self, oid, name, args, overrider, param, default):
+    def _access_attr(self, obj, name, args, overrider, param, default):
         if is_py3k:
             if type(name) is bytes:
                 name = str(name, "utf8")
@@ -585,90 +532,98 @@ class Connection(object):
             if type(name) not in (str, unicode):
                 raise TypeError("name must be a string")
             name = str(name) # IronPython issue #10 + py3k issue
-        obj = self._local_objects[oid]
         accessor = getattr(type(obj), overrider, None)
         if accessor is None:
-            name2 = self._check_attr(obj, name)
-            if not self._config[param] or not name2:
-                raise AttributeError("cannot access %r" % (name,))
             accessor = default
-            name = name2
+            name = self._check_attr(obj, name, param)
         return accessor(obj, name, *args)
 
     #
     # request handlers
     #
+    @classmethod
+    def _request_handlers(cls):
+        return {
+            consts.HANDLE_PING:        cls._handle_ping,
+            consts.HANDLE_CLOSE:       cls._handle_close,
+            consts.HANDLE_GETROOT:     cls._handle_getroot,
+            consts.HANDLE_GETATTR:     cls._handle_getattr,
+            consts.HANDLE_DELATTR:     cls._handle_delattr,
+            consts.HANDLE_SETATTR:     cls._handle_setattr,
+            consts.HANDLE_CALL:        cls._handle_call,
+            consts.HANDLE_CALLATTR:    cls._handle_callattr,
+            consts.HANDLE_REPR:        cls._handle_repr,
+            consts.HANDLE_STR:         cls._handle_str,
+            consts.HANDLE_CMP:         cls._handle_cmp,
+            consts.HANDLE_HASH:        cls._handle_hash,
+            consts.HANDLE_DIR:         cls._handle_dir,
+            consts.HANDLE_PICKLE:      cls._handle_pickle,
+            consts.HANDLE_DEL:         cls._handle_del,
+            consts.HANDLE_INSPECT:     cls._handle_inspect,
+            consts.HANDLE_BUFFITER:    cls._handle_buffiter,
+            consts.HANDLE_OLDSLICING:  cls._handle_oldslicing,
+            consts.HANDLE_CTXEXIT:     cls._handle_ctxexit,
+        }
+
     def _handle_ping(self, data):
         return data
     def _handle_close(self):
         self._cleanup()
     def _handle_getroot(self):
         return self._local_root
-    def _handle_del(self, oid, count=1):
-        self._local_objects.decref(oid)
-    def _handle_repr(self, oid):
-        return repr(self._local_objects[oid])
-    def _handle_str(self, oid):
-        return str(self._local_objects[oid])
-    def _handle_cmp(self, oid, other):
+    def _handle_del(self, obj, count=1):
+        self._local_objects.decref(id(obj), count)
+    def _handle_repr(self, obj):
+        return repr(obj)
+    def _handle_str(self, obj):
+        return str(obj)
+    def _handle_cmp(self, obj, other):
         # cmp() might enter recursive resonance... yet another workaround
-        #return cmp(self._local_objects[oid], other)
-        obj = self._local_objects[oid]
+        #return cmp(obj, other)
         try:
             return type(obj).__cmp__(obj, other)
         except (AttributeError, TypeError):
             return NotImplemented
-    def _handle_hash(self, oid):
-        return hash(self._local_objects[oid])
-    def _handle_call(self, oid, args, kwargs=()):
-        return self._local_objects[oid](*args, **dict(kwargs))
-    def _handle_dir(self, oid):
-        return tuple(dir(self._local_objects[oid]))
+    def _handle_hash(self, obj):
+        return hash(obj)
+    def _handle_call(self, obj, args, kwargs=()):
+        return obj(*args, **dict(kwargs))
+    def _handle_dir(self, obj):
+        return tuple(dir(obj))
     def _handle_inspect(self, oid):
         return tuple(netref.inspect_methods(self._local_objects[oid]))
-    def _handle_getattr(self, oid, name):
-        return self._access_attr(oid, name, (), "_rpyc_getattr", "allow_getattr", getattr)
-    def _handle_delattr(self, oid, name):
-        return self._access_attr(oid, name, (), "_rpyc_delattr", "allow_delattr", delattr)
-    def _handle_setattr(self, oid, name, value):
-        return self._access_attr(oid, name, (value,), "_rpyc_setattr", "allow_setattr", setattr)
-    def _handle_callattr(self, oid, name, args, kwargs):
-        return self._handle_getattr(oid, name)(*args, **dict(kwargs))
-    def _handle_pickle(self, oid, proto):
+    def _handle_getattr(self, obj, name):
+        return self._access_attr(obj, name, (), "_rpyc_getattr", "allow_getattr", getattr)
+    def _handle_delattr(self, obj, name):
+        return self._access_attr(obj, name, (), "_rpyc_delattr", "allow_delattr", delattr)
+    def _handle_setattr(self, obj, name, value):
+        return self._access_attr(obj, name, (value,), "_rpyc_setattr", "allow_setattr", setattr)
+    def _handle_callattr(self, obj, name, args, kwargs=()):
+        obj = self._handle_getattr(obj, name)
+        return self._handle_call(obj, args, kwargs)
+    def _handle_ctxexit(self, obj, exc):
+        if exc:
+            try:
+                raise exc
+            except:
+                exc, typ, tb = sys.exc_info()
+        else:
+            typ = tb = None
+        return self._handle_getattr(obj, "__exit__")(exc, typ, tb)
+    def _handle_pickle(self, obj, proto):
         if not self._config["allow_pickle"]:
             raise ValueError("pickling is disabled")
-        return pickle.dumps(self._local_objects[oid], proto)
-    def _handle_buffiter(self, oid, count):
-        items = []
-        obj = self._local_objects[oid]
-        i = 0
-        try:
-            while i < count:
-                items.append(next(obj))
-                i += 1
-        except StopIteration:
-            pass
-        return tuple(items)
-    def _handle_oldslicing(self, oid, attempt, fallback, start, stop, args):
+        return bytes(pickle.dumps(obj, proto))
+    def _handle_buffiter(self, obj, count):
+        return tuple(itertools.islice(obj, count))
+    def _handle_oldslicing(self, obj, attempt, fallback, start, stop, args):
         try:
             # first try __xxxitem__
-            getitem = self._handle_getattr(oid, attempt)
+            getitem = self._handle_getattr(obj, attempt)
             return getitem(slice(start, stop), *args)
         except Exception:
             # fallback to __xxxslice__. see issue #41
             if stop is None:
                 stop = maxint
-            getslice = self._handle_getattr(oid, fallback)
+            getslice = self._handle_getattr(obj, fallback)
             return getslice(start, stop, *args)
-
-    # collect handlers
-    _HANDLERS = {}
-    for name, obj in dict(locals()).items():
-        if name.startswith("_handle_"):
-            name2 = "HANDLE_" + name[8:].upper()
-            if hasattr(consts, name2):
-                _HANDLERS[getattr(consts, name2)] = obj
-            else:
-                raise NameError("no constant defined for %r", name)
-    del name, name2, obj
-
