@@ -1,11 +1,10 @@
 import rpyc
 import time
-import threading
 import sys
 import os
 import unittest
 from rpyc.utils.server import ThreadedServer
-from rpyc import SlaveService
+from rpyc import SlaveService, MasterService
 from plumbum import SshMachine
 
 
@@ -16,13 +15,10 @@ class Test_Ssh(unittest.TestCase):
             os.environ["HOME"] = os.path.expanduser("~")
             self.remote_machine = SshMachine("localhost")
         else:
-            # assume "ssh localhost" is configured to run without asking for password 
-            self.server = ThreadedServer(SlaveService, hostname = "localhost", 
+            # assume "ssh localhost" is configured to run without asking for password
+            self.server = ThreadedServer(SlaveService, hostname = "localhost",
                 ipv6 = False, port = 18888, auto_register=False)
-            t = threading.Thread(target=self.server.start)
-            t.setDaemon(True)
-            t.start()
-            time.sleep(0.5)
+            self.server._start_in_thread()
             self.remote_machine = SshMachine("localhost")
 
     def tearDown(self):
@@ -36,11 +32,9 @@ class Test_Ssh(unittest.TestCase):
         conn.modules.sys.stdout.flush()
 
     def test_connect(self):
-        conn2 = rpyc.ssh_connect(self.remote_machine, 18888, service=SlaveService)
+        conn2 = rpyc.ssh_connect(self.remote_machine, 18888, service=MasterService)
         conn2.modules.sys.stdout.write("hello through rpyc.ssh_connect()\n")
         conn2.modules.sys.stdout.flush()
 
 if __name__ == "__main__":
     unittest.main()
-
-
