@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-import threading
 import time
 import unittest
 import rpyc
@@ -7,12 +6,10 @@ import rpyc
 class MyService(rpyc.Service):
     class exposed_Invoker(object):
         def __init__(self, callback, interval):
-            self.callback = rpyc.async(callback)
+            self.callback = rpyc.async_(callback)
             self.interval = interval
             self.active = True
-            self.thread = threading.Thread(target=self.work)
-            self.thread.setDaemon(True)
-            self.thread.start()
+            self.thread = rpyc.spawn(self.work)
 
         def exposed_stop(self):
             self.active = False
@@ -52,6 +49,3 @@ class Test_Multithreaded(unittest.TestCase):
 
 if __name__ == "__main__":
     unittest.main()
-
-
-
